# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2025 Greenbone AG


import os
from argparse import ArgumentParser, ArgumentTypeError, Namespace
from pathlib import Path

from troubadix.argparser import directory_type_existing

from .models import Feed


def feed_type(value: str) -> Feed:
    try:
        return Feed[value.upper()]
    except KeyError:
        raise ArgumentTypeError(f"Invalid Feed value: '{value}'")


def parse_args() -> Namespace:
    parser = ArgumentParser(
        description="Tool for analysing the dependencies in the NASL repository.",
    )
    parser.add_argument(
        "-r",
        "--root",
        type=directory_type_existing,
        help="root for nasl directory that should be linted, uses $VTDIR if no path is given",
    )
    parser.add_argument(
        "-f",
        "--feed",
        type=feed_type,
        choices=Feed,
        nargs="+",
        default=[Feed.FULL],
        help="feed",
    )
    parser.add_argument(
        "--log",
        default="WARNING",
        help="Set the logging level (INFO, WARNING, ERROR)",
    )
    parser.add_argument("-v", "--verbose", action="count", default=0)

    args = parser.parse_args()

    if not args.root:
        vtdir = os.environ.get("VTDIR")
        if not vtdir:
            raise ValueError(
                "The environment variable 'VTDIR' is not set,"
                " and no root path with '--root' was provided."
            )
        args.root = Path(vtdir)

    return args



# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2025 Greenbone AG


from collections import Counter

import networkx as nx

from .models import Result, Script


def check_duplicates(scripts: list[Script]) -> Result:
    """
    checks for a script depending on a script multiple times
    """
    warnings = []
    for script in scripts:
        counter = Counter(dep.name for dep in script.dependencies)
        duplicates = [dep for dep, count in counter.items() if count > 1]

        if duplicates:
            msg = f"Duplicate dependencies in {script.name}: {', '.join(duplicates)}"
            warnings.append(msg)

    return Result(name="check_duplicates", warnings=warnings)


def check_missing_dependencies(
    scripts: list[Script], graph: nx.DiGraph
) -> Result:
    """
    Checks if any scripts that are depended on are missing from
    the list of scripts created from the local file system,
    logs the scripts dependending on the missing script
    """
    errors = []
    dependencies = {
        dep.name for script in scripts for dep in script.dependencies
    }
    script_names = {script.name for script in scripts}
    missing_dependencies = dependencies - script_names

    for missing in missing_dependencies:
        depending_scripts = graph.predecessors(missing)
        msg = f"missing dependency file: {missing}:"
        for script in depending_scripts:
            msg += f"\n  - used by: {script}"
        errors.append(msg)

    return Result(name="missing_dependencies", errors=errors)


def check_cycles(graph) -> Result:
    """
    checks for cyclic dependencies
    """
    if nx.is_directed_acyclic_graph(graph):
        return Result(name="check_cycles")

    cycles = nx.simple_cycles(graph)

    errors = [f"cyclic dependency: {cycle}" for cycle in cycles]
    return Result(name="check_cycles", errors=errors)


def cross_feed_dependencies(
    graph, is_enterprise_checked: bool
) -> list[tuple[str, str]]:
    """
    creates a list of script and dependency for scripts
    in community feed that depend on scripts in enterprise folders
    """
    cross_feed_dependencies = [
        (u, v)
        for u, v, is_enterprise_feed in graph.edges.data("is_enterprise_feed")
        if graph.nodes[u]["feed"] == "community"
        and graph.nodes[v].get("feed", "unknown") == "enterprise"
        and is_enterprise_feed == is_enterprise_checked
    ]  # unknown as standard value due to non existent nodes not having a feed value
    return cross_feed_dependencies


def check_cross_feed_dependencies(graph) -> Result:
    """
    Checks if scripts in the community feed have dependencies to enterprise scripts,
    and if they are correctly contained within a is_enterprise_feed check.
    """
    gated_cfd = cross_feed_dependencies(graph, is_enterprise_checked=True)
    warnings = [
        f"cross-feed-dependency: {dependent}(community feed) "
        f"depends on {dependency}(enterprise feed)"
        for dependent, dependency in gated_cfd
    ]

    ungated_cfd = cross_feed_dependencies(graph, is_enterprise_checked=False)
    errors = [
        f"unchecked cross-feed-dependency: {dependent}(community feed) "
        f"depends on {dependency}(enterprise feed), but the current feed is not properly checked"
        for dependent, dependency in ungated_cfd
    ]

    return Result(
        name="check_cross_feed_dependencies", warnings=warnings, errors=errors
    )


def check_category_order(graph) -> Result:
    problematic_edges = [
        (dependent, dependency)
        for dependent, dependency in graph.edges()
        if graph.nodes[dependent]["category"]
        < graph.nodes[dependency].get("category", -1)
    ]

    errors = [
        f"{dependent} depends on {dependency} which has a lower category order"
        for dependent, dependency in problematic_edges
    ]
    return Result(name="check_category_order", errors=errors)


def check_deprecated_dependencies(graph) -> Result:
    errors = [
        f"{dependent} depends on deprecated script {dependency}"
        for dependent, dependency in graph.edges()
        if graph.nodes[dependency].get("deprecated", False)
    ]
    return Result(name="check_deprecated_dependencies", errors=errors)



# SPDX-License-Identifier: GPL-3.0-or-later
# SPDX-FileCopyrightText: 2025 Greenbone AG

from dataclasses import dataclass, field
from enum import Flag, auto


class Feed(Flag):
    COMMON = auto()
    FEED_21_04 = auto()
    FEED_22_04 = auto()
    FULL = COMMON | FEED_21_04 | FEED_22_04

    def __str__(self):
        # Make enum values user-friendly for argparse help
        return self.name.lower()


@dataclass
class Dependency:
    name: str
    # Indicates whether the dependency will only run if an enterprise feed is used.
    # Controlled by a specific if check. Does not indicate the script's feed.
    is_enterprise_feed: bool


@dataclass
class Script:
    name: str
    feed: str
    dependencies: list[Dependency]
    category: int
    deprecated: bool


@dataclass
class Result:
    name: str
    warnings: list[str] = field(default_factory=list)
    errors: list[str] = field(default_factory=list)
